from unittest import TestCase
from src.influx_line import InfluxLine


class InfluxLineTestCase(TestCase):
    def setUp(self):
        self.measure = "weather"
        self.sample_time_stamp = "1465839830100400200"
        self.line_without_timestamp = self.complete_line = "weather,"\
            "location=us-midwest,season=summer" \
            " " \
            'temperature=82i,error=0.1,time_zone="CDT"'

        self.complete_line = self.line_without_timestamp + \
            " " \
            f"{self.sample_time_stamp}"

        self.line_with_none_field = "weather,"\
            "location=us-midwest,season=summer" \
            " " \
            'temperature="",error="",time_zone=""'

    def test_get_line_with_timestamp(self):
        """Test InfluxLine build produce right line,
         when containing timestamp """
        line = InfluxLine("weather")
        line.add_tag("location", "us-midwest")
        line.add_tag("season", "summer")
        line.add_field("temperature", 82, is_integer=True)
        line.add_field("error", 0.1)
        line.add_field("time_zone", "CDT")
        line.set_timestamp(self.sample_time_stamp)
        self.assertEqual(str(line), self.complete_line)

    def test_get_line_without_timestamp(self):
        """Test InfluxLine build produce right line,
         when containing not timestamp"""
        line = InfluxLine("weather")
        line.add_tag("location", "us-midwest")
        line.add_tag("season", "summer")
        line.add_field("temperature", 82, is_integer=True)
        line.add_field("error", 0.1)
        line.add_field("time_zone", "CDT")
        self.assertEqual(str(line), self.line_without_timestamp)

    def test_get_line_with_None_field(self):
        """Test InfluxLine build produce right line,
         when containing not timestamp"""
        line = InfluxLine("weather")
        line.add_tag("location", "us-midwest")
        line.add_tag("season", "summer")
        line.add_field("temperature", None, is_integer=True)
        line.add_field("error", None)
        line.add_field("time_zone", None)
        self.assertEqual(str(line), self.line_with_none_field)

    def test_line_with_escape_chars(self):
        """Test line with escape chars"""

        line = InfluxLine("weather")
        line.add_tag("location", "us-midwest")
        line.add_field("temp=rature", 82)
        line.set_timestamp(1465839830100400200)

        expected = "weather,location=us-midwest temp\=rature=82 1465839830100400200"
        self.assertEqual(str(line), expected)

    def test_line_with_escape_chars(self):
        """Test line with escape space"""

        line = InfluxLine("weather")
        line.add_tag("location", "us-midwest")
        line.add_field("lowest temp", 82)
        line.set_timestamp(1465839830100400200)

        expected = "weather,location=us-midwest lowest\ temp=82 1465839830100400200"
        self.assertEqual(str(line), expected)



class InfluxLine():
    """Influx line protocol builder.

    InfluxLine allow step by step building of valid Influx line protocol.

    Typical usage example:

    line = InfluxLine("weather")
    line.add_tag("location", "CA")
    line.add_field("temperature", 82.1)
    """

    def __init__(self, measure):
        self._measure = measure

    _tags = ""
    _fields = ""
    _timestamp = ""

    _escape_list = [" ", ",", "=", '"']

    def _escape(self, value):
        for char in self._escape_list:
            value = value.replace(char, f'\{char}')
        return value

    def set_measure(self, measure):
        """Sets measurement name.

        Set measure name for the InfluxLine object. Calling it
        subsequently overwrite the measure value

        Args:
        measure:
            the name of the measurement.

        Returns:
        None
        """
        self._measure = self._escape(measure)

    def set_timestamp(self, timestamp):
        """Sets timestamp.

        Set timestamp value for the InfluxLine object. Calling it
        subsequently overwrite the measure value

        Args:
        timestamp:
            the value of the measurement in nanoseconds

        Returns:
        None
        """
        self._timestamp = timestamp

    def add_tag(self, name, value):
        """add a tag to influx line.

        Adds a name and value of tag for the InfluxLine object.

        Args:
        name:
            the name of the tag

        value:
            the value of the tag

        Returns:
        None
        """
        self._tags = self._tags + \
            f",{self._escape(name)}={self._escape(str(value))}"

    def add_field(self, name, value, is_integer: bool = False):
        """add a field to influx line.

        Adds a name and value of field for the InfluxLine object.

        Args:
        name:
            the name of the field

        value:
            the value of the field

        Returns:
        None
        """
        if self._fields != "":
            self._fields = f"{self._fields},"
        if value is None:
            is_integer = False
            value = ''

        if isinstance(value, int) or isinstance(value, float):
            if is_integer:
                tag_fragment = self._create_tag_fragment(name, int(value))
                self._fields = self._fields+f"{tag_fragment}i"
            else:
                tag_fragment = self._create_tag_fragment(name, value)
                self._fields = self._fields+tag_fragment
        else:
            self._fields = self._fields + \
                f'{self._escape(name)}="{self._escape(value)}"'

    def _create_tag_fragment(self, name, value):
        tag_fragment = f"{self._escape(name)}={str(value)}"
        return tag_fragment

    def __str__(self):
        return f"{self._measure}"\
            f"{self._tags}" \
            " " \
            f"{self._fields}" \
            f"{'' if self._timestamp == '' else ' '+str(self._timestamp)}"



from setuptools import setup
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

version = os.environ.get('VERSION')
DESCRIPTION = 'Influx line protocol builder'

# Setting up
setup(
    name="influx-line",
    version=version,
    author="Oluwatimilehin Akogun",
    author_email="hello@timiakogun.com",
    description=DESCRIPTION,
    py_modules=["influx_line"],
    package_dir={'': 'src'},
    long_description_content_type="text/markdown",
    long_description=long_description,
    install_requires=[],
    keywords=['python', 'influx', 'influxdb',
              'line protocol', 'flux', 'influx line protocol'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
    ],
    extras_require={
        "dev": [
            "pytest>=3.7",
        ]
    }

)
